/**  
 * @defgroup wbxml WBXML Library
 * Result is libwbxml2
 */
 
/** 
 * @defgroup wbxml_base64 WBXML Base64 Functions
 * @ingroup wbxml
 */

/** 
 * @defgroup wbxml_buffers WBXML Buffers
 * @ingroup wbxml
 */

/** 
 * @defgroup wbxml_charset WBXML Charset
 * @ingroup wbxml
 */

/** 
 * @defgroup wbxml_conv WBXML Converters
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_elt WBXML Element
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_encoder WBXML Encoder
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_errors WBXML Error Codes
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_lists WBXML Lists
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_log WBXML Log Functions
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_mem WBXML Memory Wrapper
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_parser WBXML Parser
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_tables WBXML Tables
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml_tree WBXML Tree
 * @ingroup wbxml
 */
 
/** 
 * @defgroup wbxml2xml_tool WBXML to XML Converter Tool
 * Result is wbxml2xml
 */
 
/** 
 * @defgroup xml2wbxml_tool XML to WBXML Converter Tool
 * Result is xml2wbxml
 */
 
/**
 * @mainpage WBXML Library Documentation
 *
 * <center>
 *      Use the menu to browse the documentation [ <a href="http://libwbxml.aymerick.com/" title="WBXML Library">Back to WBXML Library Homepage</a> ]
 * </center>
 */