EXPORTS
	wbxml_attribute_create @ 1 NONAME
	wbxml_attribute_destroy @ 2 NONAME
	wbxml_attribute_duplicate @ 3 NONAME
	wbxml_attribute_get_xml_name @ 4 NONAME
	wbxml_attribute_get_xml_value @ 5 NONAME
	wbxml_attribute_name_create @ 6 NONAME
	wbxml_attribute_name_create_literal @ 7 NONAME
	wbxml_attribute_name_create_token @ 8 NONAME
	wbxml_attribute_name_destroy @ 9 NONAME
	wbxml_attribute_name_duplicate @ 10 NONAME
	wbxml_attribute_name_get_xml_name @ 11 NONAME
	wbxml_base64_decode @ 12 NONAME
	wbxml_base64_encode @ 13 NONAME
	wbxml_buffer_append @ 14 NONAME
	wbxml_buffer_append_char @ 15 NONAME
	wbxml_buffer_append_cstr_real @ 16 NONAME
	wbxml_buffer_append_data_real @ 17 NONAME
	wbxml_buffer_append_mb_uint_32 @ 18 NONAME
	wbxml_buffer_binary_to_hex @ 19 NONAME
	wbxml_buffer_compare @ 20 NONAME
	wbxml_buffer_compare_cstr @ 21 NONAME
	wbxml_buffer_contains_only_whitespaces @ 22 NONAME
	wbxml_buffer_create_real @ 23 NONAME
	wbxml_buffer_delete @ 24 NONAME
	wbxml_buffer_destroy @ 25 NONAME
	wbxml_buffer_destroy_item @ 26 NONAME
	wbxml_buffer_duplicate @ 27 NONAME
	wbxml_buffer_get_char @ 28 NONAME
	wbxml_buffer_get_cstr @ 29 NONAME
	wbxml_buffer_hex_to_binary @ 30 NONAME
	wbxml_buffer_insert @ 31 NONAME
	wbxml_buffer_insert_cstr @ 32 NONAME
	wbxml_buffer_len @ 33 NONAME
	wbxml_buffer_remove_trailing_zeros @ 34 NONAME
	wbxml_buffer_search @ 35 NONAME
	wbxml_buffer_search_char @ 36 NONAME
	wbxml_buffer_search_cstr @ 37 NONAME
	wbxml_buffer_set_char @ 38 NONAME
	wbxml_buffer_shrink_blanks @ 39 NONAME
	wbxml_buffer_split_words_real @ 40 NONAME
	wbxml_buffer_strip_blanks @ 41 NONAME
	wbxml_conv_wbxml2xml_withlen @ 42 NONAME
	wbxml_conv_xml2wbxml_withlen @ 43 NONAME
	wbxml_encoder_create_real @ 44 NONAME
	wbxml_encoder_destroy @ 45 NONAME
	wbxml_encoder_encode_to_wbxml @ 46 NONAME
	wbxml_encoder_encode_to_xml @ 47 NONAME
	wbxml_encoder_set_ignore_empty_text @ 48 NONAME
	wbxml_encoder_set_indent @ 49 NONAME
	wbxml_encoder_set_remove_text_blanks @ 50 NONAME
	wbxml_encoder_set_tree @ 51 NONAME
	wbxml_encoder_set_use_strtbl @ 52 NONAME
	wbxml_encoder_set_wbxml_version @ 53 NONAME
	wbxml_encoder_set_xml_gen_type @ 54 NONAME
	wbxml_errors_string @ 55 NONAME
	wbxml_free @ 56 NONAME
	wbxml_list_append @ 57 NONAME
	wbxml_list_create_real @ 58 NONAME
	wbxml_list_destroy @ 59 NONAME
	wbxml_list_extract_first @ 60 NONAME
	wbxml_list_get @ 61 NONAME
	wbxml_list_insert @ 62 NONAME
	wbxml_list_len @ 63 NONAME
	wbxml_malloc @ 64 NONAME
	wbxml_parser_create @ 65 NONAME
	wbxml_parser_destroy @ 66 NONAME
	wbxml_parser_get_current_byte_index @ 67 NONAME
	wbxml_parser_get_wbxml_public_id @ 68 NONAME
	wbxml_parser_get_wbxml_version @ 69 NONAME
	wbxml_parser_get_xml_public_id @ 70 NONAME
	wbxml_parser_parse @ 71 NONAME
	wbxml_parser_set_content_handler @ 72 NONAME
	wbxml_parser_set_language @ 73 NONAME
	wbxml_parser_set_main_table @ 74 NONAME
	wbxml_parser_set_user_data @ 75 NONAME
	wbxml_realloc @ 76 NONAME
	wbxml_strdup @ 77 NONAME
	wbxml_tables_contains_attr_value_from_xml @ 78 NONAME
	wbxml_tables_get_attr_from_xml @ 79 NONAME
	wbxml_tables_get_ext_from_xml @ 80 NONAME
	wbxml_tables_get_main @ 81 NONAME
	wbxml_tables_get_tag_from_xml @ 82 NONAME
	wbxml_tables_get_wbxml_publicid @ 83 NONAME
	wbxml_tables_get_xmlns @ 84 NONAME
	wbxml_tables_search_table @ 85 NONAME
	wbxml_tag_create @ 86 NONAME
	wbxml_tag_create_literal @ 87 NONAME
	wbxml_tag_create_token @ 88 NONAME
	wbxml_tag_destroy @ 89 NONAME
	wbxml_tag_duplicate @ 90 NONAME
	wbxml_tag_get_xml_name @ 91 NONAME
	wbxml_tree_attribute_create @ 92 NONAME
	wbxml_tree_attribute_destroy @ 93 NONAME
	wbxml_tree_create @ 94 NONAME
	wbxml_tree_destroy @ 95 NONAME
	wbxml_tree_from_wbxml @ 96 NONAME
	wbxml_tree_from_xml @ 97 NONAME
	wbxml_tree_node_elt_get_from_name @ 98 NONAME
	wbxml_tree_node_create @ 99 NONAME
	wbxml_tree_node_destroy @ 100 NONAME
	wbxml_tree_add_node @ 101 NONAME
	wbxml_tree_node_get_syncml_data_type @ 102 NONAME
